# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.object_storage import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ObjectACLsNegativeTest(base.BaseObjectTest):
    """Negative tests of object ACLs"""

    credentials = [['operator', CONF.object_storage.operator_role],
                   ['operator_alt', CONF.object_storage.operator_role]]

    @classmethod
    def setup_credentials(cls):
        super(ObjectACLsNegativeTest, cls).setup_credentials()
        cls.os_operator = cls.os_roles_operator_alt

    @classmethod
    def resource_setup(cls):
        super(ObjectACLsNegativeTest, cls).resource_setup()
        cls.test_auth_data = cls.os_operator.auth_provider.auth_data

    def setUp(self):
        super(ObjectACLsNegativeTest, self).setUp()
        self.container_name = data_utils.rand_name(name='TestContainer')
        self.container_client.update_container(self.container_name)

    def tearDown(self):
        self.delete_containers([self.container_name])
        super(ObjectACLsNegativeTest, self).tearDown()

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('af587587-0c24-4e15-9822-8352ce711013')
    def test_write_object_without_using_creds(self):
        """Test writing object without using credentials"""
        # trying to create object with empty headers
        # X-Auth-Token is not provided
        object_name = data_utils.rand_name(name='Object')
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=None
        )
        self.assertRaises(lib_exc.Unauthorized,
                          self.object_client.create_object,
                          self.container_name, object_name, 'data', headers={})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('af85af0b-a025-4e72-a90e-121babf55720')
    def test_delete_object_without_using_creds(self):
        """Test deleting object without using credentials"""
        # create object
        object_name = data_utils.rand_name(name='Object')
        self.object_client.create_object(self.container_name, object_name,
                                         'data')
        # trying to delete object with empty headers
        # X-Auth-Token is not provided
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=None
        )
        self.assertRaises(lib_exc.Unauthorized,
                          self.object_client.delete_object,
                          self.container_name, object_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('63d84e37-55a6-42e2-9e5f-276e60e26a00')
    def test_write_object_with_non_authorized_user(self):
        """Test writing object with non-authorized user"""
        # User provided token is forbidden. ACL are not set
        object_name = data_utils.rand_name(name='Object')
        # trying to create object with non-authorized user
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.test_auth_data
        )
        self.assertRaises(lib_exc.Forbidden,
                          self.object_client.create_object,
                          self.container_name, object_name, 'data', headers={})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('abf63359-be52-4feb-87dd-447689fc77fd')
    def test_read_object_with_non_authorized_user(self):
        """Test reading object with non-authorized user"""
        # User provided token is forbidden. ACL are not set
        object_name = data_utils.rand_name(name='Object')
        resp, _ = self.object_client.create_object(
            self.container_name, object_name, 'data')
        self.assertHeaders(resp, 'Object', 'PUT')
        # trying to get object with non authorized user token
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.test_auth_data
        )
        self.assertRaises(lib_exc.Forbidden,
                          self.object_client.get_object,
                          self.container_name, object_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7343ac3d-cfed-4198-9bb0-00149741a492')
    def test_delete_object_with_non_authorized_user(self):
        """Test deleting object with non-authorized user"""
        # User provided token is forbidden. ACL are not set
        object_name = data_utils.rand_name(name='Object')
        resp, _ = self.object_client.create_object(
            self.container_name, object_name, 'data')
        self.assertHeaders(resp, 'Object', 'PUT')
        # trying to delete object with non-authorized user token
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.test_auth_data
        )
        self.assertRaises(lib_exc.Forbidden,
                          self.object_client.delete_object,
                          self.container_name, object_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9ed01334-01e9-41ea-87ea-e6f465582823')
    def test_read_object_without_rights(self):
        """Test reading object without rights"""
        # update X-Container-Read metadata ACL
        cont_headers = {'X-Container-Read': 'badtenant:baduser'}
        resp_meta, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                self.container_name, create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')
        # create object
        object_name = data_utils.rand_name(name='Object')
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, 'data')
        self.assertHeaders(resp, 'Object', 'PUT')
        # Trying to read the object without rights
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.test_auth_data
        )
        self.assertRaises(lib_exc.Forbidden,
                          self.object_client.get_object,
                          self.container_name, object_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a3a585a7-d8cf-4b65-a1a0-edc2b1204f85')
    def test_write_object_without_rights(self):
        """Test writing object without rights"""
        # update X-Container-Write metadata ACL
        cont_headers = {'X-Container-Write': 'badtenant:baduser'}
        resp_meta, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                self.container_name, create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')
        # Trying to write the object without rights
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.test_auth_data
        )
        object_name = data_utils.rand_name(name='Object')
        self.assertRaises(lib_exc.Forbidden,
                          self.object_client.create_object,
                          self.container_name,
                          object_name, 'data', headers={})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8ba512ad-aa6e-444e-b882-2906a0ea2052')
    def test_write_object_without_write_rights(self):
        """Test writing object without write rights"""
        # update X-Container-Read and X-Container-Write metadata ACL
        tenant_name = self.os_operator.credentials.tenant_name
        username = self.os_operator.credentials.username
        cont_headers = {'X-Container-Read':
                        tenant_name + ':' + username,
                        'X-Container-Write': ''}
        resp_meta, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                self.container_name, create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')
        # Trying to write the object without write rights
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.test_auth_data
        )
        object_name = data_utils.rand_name(name='Object')
        self.assertRaises(lib_exc.Forbidden,
                          self.object_client.create_object,
                          self.container_name,
                          object_name, 'data', headers={})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b4e366f8-f185-47ab-b789-df4416f9ecdb')
    def test_delete_object_without_write_rights(self):
        """Test deleting object without write rights"""
        # update X-Container-Read and X-Container-Write metadata ACL
        tenant_name = self.os_operator.credentials.tenant_name
        username = self.os_operator.credentials.username
        cont_headers = {'X-Container-Read':
                        tenant_name + ':' + username,
                        'X-Container-Write': ''}
        resp_meta, _ = (
            self.container_client.create_update_or_delete_container_metadata(
                self.container_name, create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')
        # create object
        object_name = data_utils.rand_name(name='Object')
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, 'data')
        self.assertHeaders(resp, 'Object', 'PUT')
        # Trying to delete the object without write rights
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.test_auth_data
        )
        self.assertRaises(lib_exc.Forbidden,
                          self.object_client.delete_object,
                          self.container_name,
                          object_name)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ServicesTestJSON(base.BaseIdentityV2AdminTest):
    """Test identity services via v2 API"""

    def _del_service(self, service_id):
        # Deleting the service created in this method
        self.services_client.delete_service(service_id)
        # Checking whether service is deleted successfully
        self.assertRaises(lib_exc.NotFound, self.services_client.show_service,
                          service_id)

    @decorators.idempotent_id('84521085-c6e6-491c-9a08-ec9f70f90110')
    def test_create_get_delete_service(self):
        """Test verifies the identity service create/get/delete via v2 API"""
        # GET Service
        # Creating a Service
        name = data_utils.rand_name('service')
        s_type = data_utils.rand_name('type')
        description = data_utils.rand_name('description')
        service_data = self.services_client.create_service(
            name=name, type=s_type,
            description=description)['OS-KSADM:service']
        self.assertIsNotNone(service_data['id'])
        self.addCleanup(self._del_service, service_data['id'])
        # Verifying response body of create service
        self.assertIn('name', service_data)
        self.assertEqual(name, service_data['name'])
        self.assertIn('type', service_data)
        self.assertEqual(s_type, service_data['type'])
        self.assertIn('description', service_data)
        self.assertEqual(description, service_data['description'])
        # Get service
        fetched_service = (
            self.services_client.show_service(service_data['id'])
            ['OS-KSADM:service'])
        # verifying the existence of service created
        self.assertIn('id', fetched_service)
        self.assertEqual(fetched_service['id'], service_data['id'])
        self.assertIn('name', fetched_service)
        self.assertEqual(fetched_service['name'], service_data['name'])
        self.assertIn('type', fetched_service)
        self.assertEqual(fetched_service['type'], service_data['type'])
        self.assertIn('description', fetched_service)
        self.assertEqual(fetched_service['description'],
                         service_data['description'])

    @decorators.idempotent_id('5d3252c8-e555-494b-a6c8-e11d7335da42')
    def test_create_service_without_description(self):
        """Test creating identity service without description via v2 API

        Create a service only with name and type.
        """
        name = data_utils.rand_name('service')
        s_type = data_utils.rand_name('type')
        service = self.services_client.create_service(
            name=name, type=s_type)['OS-KSADM:service']
        self.assertIn('id', service)
        self.addCleanup(self._del_service, service['id'])
        self.assertIn('name', service)
        self.assertEqual(name, service['name'])
        self.assertIn('type', service)
        self.assertEqual(s_type, service['type'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('34ea6489-012d-4a86-9038-1287cadd5eca')
    def test_list_services(self):
        """Test Create/List/Verify/Delete of identity service via v2 API"""
        services = []
        for _ in range(3):
            name = data_utils.rand_name('service')
            s_type = data_utils.rand_name('type')
            description = data_utils.rand_name('description')

            service = self.services_client.create_service(
                name=name, type=s_type,
                description=description)['OS-KSADM:service']
            self.addCleanup(self.services_client.delete_service, service['id'])
            services.append(service)
        service_ids = [svc['id'] for svc in services]

        # List and Verify Services
        body = self.services_client.list_services()['OS-KSADM:services']
        found = [serv for serv in body if serv['id'] in service_ids]
        self.assertEqual(len(found), len(services), 'Services not found')



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest.lib.services.compute.agents_client import AgentsClient
from tempest.lib.services.compute.aggregates_client import AggregatesClient
from tempest.lib.services.compute.assisted_volume_snapshots_client import \
    AssistedVolumeSnapshotsClient
from tempest.lib.services.compute.availability_zone_client import \
    AvailabilityZoneClient
from tempest.lib.services.compute.baremetal_nodes_client import \
    BaremetalNodesClient
from tempest.lib.services.compute.certificates_client import \
    CertificatesClient
from tempest.lib.services.compute.extensions_client import \
    ExtensionsClient
from tempest.lib.services.compute.fixed_ips_client import FixedIPsClient
from tempest.lib.services.compute.flavors_client import FlavorsClient
from tempest.lib.services.compute.floating_ip_pools_client import \
    FloatingIPPoolsClient
from tempest.lib.services.compute.floating_ips_bulk_client import \
    FloatingIPsBulkClient
from tempest.lib.services.compute.floating_ips_client import \
    FloatingIPsClient
from tempest.lib.services.compute.hosts_client import HostsClient
from tempest.lib.services.compute.hypervisor_client import \
    HypervisorClient
from tempest.lib.services.compute.images_client import ImagesClient
from tempest.lib.services.compute.instance_usage_audit_log_client import \
    InstanceUsagesAuditLogClient
from tempest.lib.services.compute.interfaces_client import InterfacesClient
from tempest.lib.services.compute.keypairs_client import KeyPairsClient
from tempest.lib.services.compute.limits_client import LimitsClient
from tempest.lib.services.compute.migrations_client import MigrationsClient
from tempest.lib.services.compute.networks_client import NetworksClient
from tempest.lib.services.compute.quota_classes_client import \
    QuotaClassesClient
from tempest.lib.services.compute.quotas_client import QuotasClient
from tempest.lib.services.compute.security_group_default_rules_client import \
    SecurityGroupDefaultRulesClient
from tempest.lib.services.compute.security_group_rules_client import \
    SecurityGroupRulesClient
from tempest.lib.services.compute.security_groups_client import \
    SecurityGroupsClient
from tempest.lib.services.compute.server_groups_client import \
    ServerGroupsClient
from tempest.lib.services.compute.servers_client import ServersClient
from tempest.lib.services.compute.services_client import ServicesClient
from tempest.lib.services.compute.snapshots_client import SnapshotsClient
from tempest.lib.services.compute.tenant_networks_client import \
    TenantNetworksClient
from tempest.lib.services.compute.tenant_usages_client import \
    TenantUsagesClient
from tempest.lib.services.compute.versions_client import VersionsClient
from tempest.lib.services.compute.volumes_client import \
    VolumesClient

__all__ = ['AgentsClient', 'AggregatesClient', 'AssistedVolumeSnapshotsClient',
           'AvailabilityZoneClient', 'BaremetalNodesClient',
           'CertificatesClient', 'ExtensionsClient', 'FixedIPsClient',
           'FlavorsClient', 'FloatingIPPoolsClient',
           'FloatingIPsBulkClient', 'FloatingIPsClient', 'HostsClient',
           'HypervisorClient', 'ImagesClient', 'InstanceUsagesAuditLogClient',
           'InterfacesClient', 'KeyPairsClient', 'LimitsClient',
           'MigrationsClient', 'NetworksClient', 'QuotaClassesClient',
           'QuotasClient', 'SecurityGroupDefaultRulesClient',
           'SecurityGroupRulesClient', 'SecurityGroupsClient',
           'ServerGroupsClient', 'ServersClient', 'ServicesClient',
           'SnapshotsClient', 'TenantNetworksClient', 'TenantUsagesClient',
           'VersionsClient', 'VolumesClient']



# Copyright 2015 Red Hat Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class DomainsNegativeTestJSON(base.BaseIdentityV3AdminTest):
    """Negative tests of identity domains"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @decorators.attr(type=['negative', 'gate'])
    @decorators.idempotent_id('1f3fbff5-4e44-400d-9ca1-d953f05f609b')
    def test_delete_active_domain(self):
        """Test deleting active domain should fail"""
        domain = self.create_domain()
        domain_id = domain['id']

        self.addCleanup(self.delete_domain, domain_id)

        # domain need to be disabled before deleting
        self.assertRaises(lib_exc.Forbidden, self.domains_client.delete_domain,
                          domain_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9018461d-7d24-408d-b3fe-ae37e8cd5c9e')
    def test_create_domain_with_empty_name(self):
        """Test creating domain with empty name should fail

        Domain name should not be empty
        """
        self.assertRaises(lib_exc.BadRequest,
                          self.domains_client.create_domain, name='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('37b1bbf2-d664-4785-9a11-333438586eae')
    def test_create_domain_with_name_length_over_64(self):
        """Test creating domain with name over length

        Domain name length should not ne greater than 64 characters
        """
        d_name = 'a' * 65
        self.assertRaises(lib_exc.BadRequest,
                          self.domains_client.create_domain,
                          name=d_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('43781c07-764f-4cf2-a405-953c1916f605')
    def test_delete_non_existent_domain(self):
        """Test attempting to delete a non existent domain should fail"""
        self.assertRaises(lib_exc.NotFound, self.domains_client.delete_domain,
                          data_utils.rand_uuid_hex())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e6f9e4a2-4f36-4be8-bdbc-4e199ae29427')
    def test_domain_create_duplicate(self):
        """Test creating domain with duplicate name should fail"""
        domain_name = data_utils.rand_name('domain-dup')
        domain = self.domains_client.create_domain(name=domain_name)['domain']
        domain_id = domain['id']
        self.addCleanup(self.delete_domain, domain_id)
        # Domain name should be unique
        self.assertRaises(
            lib_exc.Conflict, self.domains_client.create_domain,
            name=domain_name)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import operator

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class VolumesListAdminTestJSON(base.BaseVolumeAdminTest):
    """Test listing volumes with admin privilege"""

    @classmethod
    def resource_setup(cls):
        super(VolumesListAdminTestJSON, cls).resource_setup()
        # Create 3 test volumes
        # NOTE(zhufl): When using pre-provisioned credentials, the project
        # may have volumes other than those created below.
        cls.volume_list = cls.volumes_client.list_volumes()['volumes']
        for _ in range(3):
            volume = cls.create_volume()
            # Fetch volume details
            volume_details = cls.volumes_client.show_volume(
                volume['id'])['volume']
            cls.volume_list.append(volume_details)

    @decorators.idempotent_id('5866286f-3290-4cfd-a414-088aa6cdc469')
    def test_volume_list_param_tenant(self):
        """Test admin can list volumes belonging to specified project"""
        # Create a volume in admin tenant
        adm_vol = self.admin_volume_client.create_volume(
            size=CONF.volume.volume_size)['volume']
        self.addCleanup(self.admin_volume_client.delete_volume, adm_vol['id'])
        waiters.wait_for_volume_resource_status(self.admin_volume_client,
                                                adm_vol['id'], 'available')
        params = {'all_tenants': 1,
                  'project_id': self.volumes_client.tenant_id}
        # Getting volume list from primary tenant using admin credentials
        fetched_list = self.admin_volume_client.list_volumes(
            detail=True, params=params)['volumes']
        # Verifying fetched volume ids list is related to primary tenant
        fetched_list_ids = map(operator.itemgetter('id'), fetched_list)
        expected_list_ids = map(operator.itemgetter('id'), self.volume_list)
        self.assertEqual(sorted(expected_list_ids), sorted(fetched_list_ids))
        # Verifying tenant id of volumes fetched list is related to
        # primary tenant
        fetched_tenant_id = [operator.itemgetter(
            'os-vol-tenant-attr:tenant_id')(item) for item in fetched_list]
        expected_tenant_id = [self.volumes_client.tenant_id] * \
            len(self.volume_list)
        self.assertEqual(expected_tenant_id, fetched_tenant_id)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib import decorators


class AbsoluteLimitsTestJSON(base.BaseV2ComputeTest):
    """Test compute absolute limits

    Test compute absolute limits with compute microversion less than 2.57
    """

    max_microversion = '2.56'

    @classmethod
    def setup_clients(cls):
        super(AbsoluteLimitsTestJSON, cls).setup_clients()
        cls.client = cls.limits_client

    @decorators.idempotent_id('b54c66af-6ab6-4cf0-a9e5-a0cb58d75e0b')
    def test_absLimits_get(self):
        """Test getting nova absolute limits"""
        # To check if all limits are present in the response (will be checked
        # by schema)
        self.client.show_limits()


class AbsoluteLimitsV257TestJSON(base.BaseV2ComputeTest):
    """Test compute absolute limits

    Test compute absolute limits with compute microversion greater than 2.56
    """
    min_microversion = '2.57'
    max_microversion = 'latest'

    # NOTE(felipemonteiro): This class tests the Absolute Limits APIs
    # response schema for the 2.57 microversion.



# Copyright 2020 Samsung Electronics Co., Ltd
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest.lib.services.identity.v3 import service_providers_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestServiceProvidersClient(base.BaseServiceTest):
    FAKE_SERVICE_PROVIDER_INFO = {
        "service_provider": {
            "auth_url": "https://example.com/identity/v3/OS-FEDERATION/" +
                        "identity_providers/FAKE_ID/protocols/fake_id1/auth",
            "description": "Fake Service Provider",
            "enabled": True,
            "id": "FAKE_ID",
            "links": {
                "self": "https://example.com/identity/v3/OS-FEDERATION/" +
                        "service_providers/FAKE_ID"
            },
            "relay_state_prefix": "ss:mem:",
            "sp_url": "https://example.com/identity/Shibboleth.sso/" +
                      "FAKE_ID1/ECP"
        }
    }

    FAKE_SERVICE_PROVIDERS_INFO = {
        "links": {
            "next": None,
            "previous": None,
            "self": "http://example.com/identity/v3/OS-FEDERATION/" +
                    "service_providers"
        },
        "service_providers": [
            {
                "auth_url": "https://example.com/identity/v3/OS-FEDERATION/" +
                            "identity_providers/acme/protocols/saml2/auth",
                "description": "Stores ACME identities",
                "enabled": True,
                "id": "ACME",
                "links": {
                    "self": "http://example.com/identity/v3/OS-FEDERATION/" +
                            "service_providers/ACME"
                },
                "relay_state_prefix": "ss:mem:",
                "sp_url": "https://example.com/identity/Shibboleth.sso/" +
                          "SAML2/ECP"
            },
            {
                "auth_url": "https://other.example.com/identity/v3/" +
                            "OS-FEDERATION/identity_providers/acme/" +
                            "protocols/saml2/auth",
                "description": "Stores contractor identities",
                "enabled": False,
                "id": "ACME-contractors",
                "links": {
                    "self": "http://example.com/identity/v3/OS-FEDERATION/" +
                            "service_providers/ACME-contractors"
                },
                "relay_state_prefix": "ss:mem:",
                "sp_url": "https://other.example.com/identity/Shibboleth" +
                          ".sso/SAML2/ECP"
            }
        ]
    }

    def setUp(self):
        super(TestServiceProvidersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = service_providers_client.ServiceProvidersClient(
            fake_auth, 'identity', 'regionOne')

    def _test_register_service_provider(self, bytes_body=False):
        self.check_service_client_function(
            self.client.register_service_provider,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_SERVICE_PROVIDER_INFO,
            bytes_body,
            service_provider_id="FAKE_ID",
            status=201)

    def _test_list_service_providers(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_service_providers,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVICE_PROVIDERS_INFO,
            bytes_body,
            status=200)

    def _test_get_service_provider(self, bytes_body=False):
        self.check_service_client_function(
            self.client.get_service_provider,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SERVICE_PROVIDER_INFO,
            bytes_body,
            service_provider_id="FAKE_ID",
            status=200)

    def _test_delete_service_provider(self, bytes_body=False):
        self.check_service_client_function(
            self.client.delete_service_provider,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            bytes_body,
            service_provider_id="FAKE_ID",
            status=204)

    def _test_update_service_provider(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_service_provider,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_SERVICE_PROVIDER_INFO,
            bytes_body,
            service_provider_id="FAKE_ID",
            status=200)

    def test_register_service_provider_with_str_body(self):
        self._test_register_service_provider()

    def test_register_service_provider_with_bytes_body(self):
        self._test_register_service_provider(bytes_body=True)

    def test_list_service_providers_with_str_body(self):
        self._test_list_service_providers()

    def test_list_service_providers_with_bytes_body(self):
        self._test_list_service_providers(bytes_body=True)

    def test_get_service_provider_with_str_body(self):
        self._test_get_service_provider()

    def test_get_service_provider_with_bytes_body(self):
        self._test_get_service_provider(bytes_body=True)

    def test_delete_service_provider_with_str_body(self):
        self._test_delete_service_provider()

    def test_delete_service_provider_with_bytes_body(self):
        self._test_delete_service_provider(bytes_body=True)

    def test_update_service_provider_with_str_body(self):
        self._test_update_service_provider()

    def test_update_service_provider_with_bytes_body(self):
        self._test_update_service_provider(bytes_body=True)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import utils
from tempest.common.utils.linux import remote_client
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class BaseAttachVolumeTest(base.BaseV2ComputeTest):
    """Base class for the attach volume tests in this module."""
    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(BaseAttachVolumeTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(BaseAttachVolumeTest, cls).setup_credentials()

    def _create_server(self):
        # Start a server and wait for it to become ready
        validation_resources = self.get_test_validation_resources(
            self.os_primary)
        server = self.create_test_server(
            validatable=True,
            validation_resources=validation_resources,
            wait_until='ACTIVE',
            adminPass=self.image_ssh_password)
        self.addCleanup(self.delete_server, server['id'])
        # Record addresses so that we can ssh later
        server['addresses'] = self.servers_client.list_addresses(
            server['id'])['addresses']
        return server, validation_resources


class AttachVolumeTestJSON(BaseAttachVolumeTest):
    """Test attaching volume to server"""

    @decorators.idempotent_id('52e9045a-e90d-4c0d-9087-79d657faffff')
    # This test is conditionally marked slow if SSH validation is enabled.
    @decorators.attr(type='slow', condition=CONF.validation.run_validation)
    def test_attach_detach_volume(self):
        """Test attaching and detaching volume from server

        Stop and Start a server with an attached volume, ensuring that
        the volume remains attached.
        """
        server, validation_resources = self._create_server()

        # NOTE(andreaf) Create one remote client used throughout the test.
        if CONF.validation.run_validation:
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.image_ssh_user,
                self.image_ssh_password,
                validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.servers_client)
            # NOTE(andreaf) We need to ensure the ssh key has been
            # injected in the guest before we power cycle
            linux_client.validate_authentication()
            disks_before_attach = linux_client.list_disks()

        volume = self.create_volume()

        # NOTE: As of the 12.0.0 Liberty release, the Nova libvirt driver
        # no longer honors a user-supplied device name, and there can be
        # a mismatch between libvirt provide disk name and actual disk name
        # on instance, hence we no longer validate this test with the supplied
        # device name rather we count number of disk before attach
        # detach to validate the testcase.

        attachment = self.attach_volume(server, volume)

        self.servers_client.stop_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'SHUTOFF')

        self.servers_client.start_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')

        if CONF.validation.run_validation:
            disks_after_attach = linux_client.list_disks()
            self.assertGreater(
                len(disks_after_attach),
                len(disks_before_attach))

        self.servers_client.detach_volume(server['id'], attachment['volumeId'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client, attachment['volumeId'], 'available')

        self.servers_client.stop_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'SHUTOFF')

        self.servers_client.start_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')

        if CONF.validation.run_validation:
            disks_after_detach = linux_client.list_disks()
            self.assertEqual(len(disks_before_attach), len(disks_after_detach))

    @decorators.idempotent_id('7fa563fe-f0f7-43eb-9e22-a1ece036b513')
    def test_list_get_volume_attachments(self):
        """Test listing and getting volume attachments

        First we attach one volume to the server, check listing and getting
        the volume attachment of the server. Then we attach another volume to
        the server, check listing and getting the volume attachments of the
        server. Finally we detach the volumes from the server one by one.
        """
        # List volume attachment of the server
        server, validation_resources = self._create_server()
        volume_1st = self.create_volume()
        attachment_1st = self.attach_volume(server, volume_1st)
        body = self.servers_client.list_volume_attachments(
            server['id'])['volumeAttachments']
        self.assertEqual(1, len(body))
        self.assertIn(attachment_1st, body)

        # Get volume attachment of the server
        body = self.servers_client.show_volume_attachment(
            server['id'],
            attachment_1st['id'])['volumeAttachment']
        self.assertEqual(server['id'], body['serverId'])
        self.assertEqual(volume_1st['id'], body['volumeId'])
        self.assertEqual(attachment_1st['id'], body['id'])

        # attach one more volume to server
        volume_2nd = self.create_volume()
        attachment_2nd = self.attach_volume(server, volume_2nd)
        body = self.servers_client.list_volume_attachments(
            server['id'])['volumeAttachments']
        self.assertEqual(2, len(body))

        if CONF.validation.run_validation:
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.image_ssh_user,
                self.image_ssh_password,
                validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.servers_client)
            linux_client.validate_authentication()

        for attachment in [attachment_1st, attachment_2nd]:
            body = self.servers_client.show_volume_attachment(
                server['id'], attachment['id'])['volumeAttachment']
            self.assertEqual(server['id'], body['serverId'])
            self.assertEqual(attachment['volumeId'], body['volumeId'])
            self.assertEqual(attachment['id'], body['id'])
            self.servers_client.detach_volume(server['id'],
                                              attachment['volumeId'])
            waiters.wait_for_volume_resource_status(
                self.volumes_client, attachment['volumeId'], 'available')


class AttachVolumeShelveTestJSON(BaseAttachVolumeTest):
    """Testing volume with shelved instance.

    This test checks the attaching and detaching volumes from
    a shelved or shelved offload instance.

    Note that these are uncommon scenarios until blueprint detach-boot-volume
    is implemented in the compute service.
    """

    min_microversion = '2.20'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(AttachVolumeShelveTestJSON, cls).skip_checks()
        if not CONF.compute_feature_enabled.shelve:
            raise cls.skipException('Shelve is not available.')
        if CONF.compute.compute_volume_common_az:
            # assuming cross_az_attach is set to false in nova.conf
            # per the compute_volume_common_az option description
            raise cls.skipException('Cross AZ attach not available.')

    def _count_volumes(self, server, validation_resources):
        # Count number of volumes on an instance
        volumes = 0
        if CONF.validation.run_validation:
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.image_ssh_user,
                self.image_ssh_password,
                validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.servers_client)

            command = 'grep -c -E [vs]d.$ /proc/partitions'
            volumes = int(linux_client.exec_command(command).strip())
        return volumes

    def _shelve_server(self, server, validation_resources):
        # NOTE(andreaf) If we are going to shelve a server, we should
        # check first whether the server is ssh-able. Otherwise we
        # won't be able to distinguish failures introduced by shelve
        # from pre-existing ones. Also it's good to wait for cloud-init
        # to be done and sshd server to be running before shelving to
        # avoid breaking the VM
        if CONF.validation.run_validation:
            linux_client = remote_client.RemoteClient(
                self.get_server_ip(server, validation_resources),
                self.image_ssh_user,
                self.image_ssh_password,
                validation_resources['keypair']['private_key'],
                server=server,
                servers_client=self.servers_client)
            linux_client.validate_authentication()

        # If validation went ok, or it was skipped, shelve the server
        compute.shelve_server(self.servers_client, server['id'])

    def _unshelve_server_and_check_volumes(self, server,
                                           validation_resources,
                                           number_of_volumes):
        # Unshelve the instance and check that there are expected volumes
        self.servers_client.unshelve_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'],
                                       'ACTIVE')
        if CONF.validation.run_validation:
            counted_volumes = self._count_volumes(
                server, validation_resources)
            self.assertEqual(number_of_volumes, counted_volumes)

    # NOTE(mriedem): Marked as slow since this is an uncommon scenario until
    # attach/detach root volume is supported in nova, and it's slow.
    @decorators.attr(type='slow')
    @decorators.idempotent_id('13a940b6-3474-4c3c-b03f-29b89112bfee')
    def test_attach_volume_shelved_or_offload_server(self):
        """Test attaching volume to shelved server

        Create server, count number of volumes on it, shelve
        server and attach pre-created volume to shelved server, then
        unshelve the server and check that attached volume exists.
        """
        server, validation_resources = self._create_server()
        volume = self.create_volume()
        num_vol = self._count_volumes(server, validation_resources)
        self._shelve_server(server, validation_resources)
        attachment = self.attach_volume(server, volume)

        # Unshelve the instance and check that attached volume exists
        self._unshelve_server_and_check_volumes(
            server, validation_resources, num_vol + 1)

        # Get volume attachment of the server
        volume_attachment = self.servers_client.show_volume_attachment(
            server['id'],
            attachment['id'])['volumeAttachment']
        self.assertEqual(server['id'], volume_attachment['serverId'])
        self.assertEqual(attachment['id'], volume_attachment['id'])
        # Check the mountpoint is not None after unshelve server even in
        # case of shelved_offloaded.
        self.assertIsNotNone(volume_attachment['device'])

    # NOTE(mriedem): Marked as slow since this is an uncommon scenario until
    # attach/detach root volume is supported in nova, and it's slow.
    @decorators.attr(type='slow')
    @decorators.idempotent_id('b54e86dd-a070-49c4-9c07-59ae6dae15aa')
    def test_detach_volume_shelved_or_offload_server(self):
        """Test detaching volume from shelved server

        Count number of volumes on server, shelve server and attach
        pre-created volume to shelved server, then detach the volume, unshelve
        the instance and check that we have the expected number of volume(s).
        """
        server, validation_resources = self._create_server()
        volume = self.create_volume()
        num_vol = self._count_volumes(server, validation_resources)
        self._shelve_server(server, validation_resources)

        # Attach and then detach the volume
        self.attach_volume(server, volume)
        self.servers_client.detach_volume(server['id'], volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        # Unshelve the instance and check that we have the expected number of
        # volume(s)
        self._unshelve_server_and_check_volumes(
            server, validation_resources, num_vol)


class AttachVolumeMultiAttachTest(BaseAttachVolumeTest):
    """Test attaching one volume to multiple servers

    Test attaching one volume to multiple servers with compute
    microversion greater than 2.59.
    """

    min_microversion = '2.60'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(AttachVolumeMultiAttachTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.volume_multiattach:
            raise cls.skipException('Volume multi-attach is not available.')

    def _attach_volume_to_servers(self, volume, servers):
        """Attaches the given volume to the list of servers.

        :param volume: The multiattach volume to use.
        :param servers: list of server instances on which the volume will be
                        attached
        :returns: dict of server ID to volumeAttachment dict entries
        """
        attachments = {}
        for server in servers:
            # map the server id to the volume attachment
            attachments[server['id']] = self.attach_volume(server, volume)
            # NOTE(mriedem): In the case of multi-attach, after the first
            # attach the volume will be in-use. On the second attach, nova will
            # 'reserve' the volume which puts it back into 'attaching' status
            # and then the volume shouldn't go back to in-use until the compute
            # actually attaches the server to the volume.
        return attachments

    def _detach_multiattach_volume(self, volume_id, server_id):
        """Detaches a multiattach volume from the given server.

        Depending on the number of attachments the volume has, this method
        will wait for the volume to go to back to 'in-use' status if there are
        more attachments or 'available' state if there are no more attachments.
        """
        # Count the number of attachments before starting the detach.
        volume = self.volumes_client.show_volume(volume_id)['volume']
        attachments = volume['attachments']
        wait_status = 'in-use' if len(attachments) > 1 else 'available'
        # Now detach the volume from the given server.
        self.servers_client.detach_volume(server_id, volume_id)
        # Now wait for the volume status to change.
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_id, wait_status)

    def _create_multiattach_volume(self, bootable=False):
        kwargs = {}
        if bootable:
            kwargs['image_ref'] = CONF.compute.image_ref
        return self.create_volume(multiattach=True, **kwargs)

    def _create_and_multiattach(self):
        """Creates two server instances and a volume and attaches to both.

        :returns: A three-item tuple of the list of created servers,
                  the created volume, and dict of server ID to volumeAttachment
                  dict entries
        """
        servers = []
        for x in range(2):
            name = 'multiattach-server-%i' % x
            servers.append(self.create_test_server(name=name))

        # Now wait for the servers to be ACTIVE.
        for server in servers:
            waiters.wait_for_server_status(self.servers_client, server['id'],
                                           'ACTIVE')

        volume = self._create_multiattach_volume()

        # Attach the volume to the servers
        attachments = self._attach_volume_to_servers(volume, servers)
        return servers, volume, attachments

    @decorators.idempotent_id('8d5853f7-56e7-4988-9b0c-48cea3c7049a')
    def test_list_get_volume_attachments_multiattach(self):
        """Test listing and getting multiattached volume attachments

        Attach a single volume to two servers, list attachments from the
        volume and make sure the server uuids are in the list, then detach
        the volume from servers one by one.
        """
        # Attach a single volume to two servers.
        servers, volume, attachments = self._create_and_multiattach()

        # List attachments from the volume and make sure the server uuids
        # are in that list.
        vol_attachments = self.volumes_client.show_volume(
            volume['id'])['volume']['attachments']
        attached_server_ids = [attachment['server_id']
                               for attachment in vol_attachments]
        self.assertEqual(2, len(attached_server_ids))

        # List Volume attachment of the servers
        for server in servers:
            self.assertIn(server['id'], attached_server_ids)
            vol_attachments = self.servers_client.list_volume_attachments(
                server['id'])['volumeAttachments']
            self.assertEqual(1, len(vol_attachments))
            attachment = attachments[server['id']]
            self.assertDictEqual(attachment, vol_attachments[0])
            # Detach the volume from this server.
            self._detach_multiattach_volume(volume['id'], server['id'])

    def _boot_from_multiattach_volume(self):
        """Boots a server from a multiattach volume.

        The volume will not be deleted when the server is deleted.

        :returns: 2-item tuple of (server, volume)
        """
        volume = self._create_multiattach_volume(bootable=True)
        # Now create a server from the bootable volume.
        bdm = [{
            'uuid': volume['id'],
            'source_type': 'volume',
            'destination_type': 'volume',
            'boot_index': 0,
            'delete_on_termination': False}]
        server = self.create_test_server(
            image_id='', block_device_mapping_v2=bdm, wait_until='ACTIVE')
        # Assert the volume is attached to the server.
        attachments = self.servers_client.list_volume_attachments(
            server['id'])['volumeAttachments']
        self.assertEqual(1, len(attachments))
        self.assertEqual(volume['id'], attachments[0]['volumeId'])
        return server, volume

    @decorators.idempotent_id('65e33aa2-185b-44c8-b22e-e524973ed625')
    def test_boot_from_multiattach_volume(self):
        """Simple test to boot an instance from a multiattach volume."""
        self._boot_from_multiattach_volume()

    @utils.services('image')
    @decorators.idempotent_id('885ac48a-2d7a-40c5-ae8b-1993882d724c')
    def test_snapshot_volume_backed_multiattach(self):
        """Boots a server from a multiattach volume and snapshots the server.

        Creating the snapshot of the server will also create a snapshot of
        the volume.
        """
        server, volume = self._boot_from_multiattach_volume()
        # Create a snapshot of the server (and volume implicitly).
        self.create_image_from_server(
            server['id'], name='multiattach-snapshot',
            wait_until='active', wait_for_server=True)
        # TODO(mriedem): Make sure the volume snapshot exists. This requires
        # adding the volume snapshots client to BaseV2ComputeTest.
        # Delete the server, wait for it to be gone, and make sure the volume
        # still exists.
        self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])
        # Delete the volume and cascade the delete of the volume snapshot.
        self.volumes_client.delete_volume(volume['id'], cascade=True)
        # Now we have to wait for the volume to be gone otherwise the normal
        # teardown will fail since it will race with our call and the snapshot
        # might still exist.
        self.volumes_client.wait_for_resource_deletion(volume['id'])

    @decorators.idempotent_id('f01c7169-a124-4fc7-ae60-5e380e247c9c')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_resize_server_with_multiattached_volume(self):
        """Test resizing servers with multiattached volume

        Attach a single volume to multiple servers, then resize the servers
        """
        servers, volume, _ = self._create_and_multiattach()

        for server in servers:
            self.resize_server(server['id'], self.flavor_ref_alt)

        for server in servers:
            self._detach_multiattach_volume(volume['id'], server['id'])

    # TODO(mriedem): Might be interesting to create a bootable multiattach
    # volume with delete_on_termination=True, create server1 from the
    # volume, then attach it to server2, and then delete server1 in which
    # case the volume won't be deleted because it's still attached to
    # server2 and make sure the volume is still attached to server2.



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import sys

import fixtures
import pkg_resources
import testtools


def _handle_skip_exception():
    try:
        stestr_version = pkg_resources.parse_version(
            pkg_resources.get_distribution("stestr").version)
        stestr_min = pkg_resources.parse_version('2.5.0')
        new_stestr = (stestr_version >= stestr_min)
        import unittest
        import unittest2
        if sys.version_info >= (3, 5) and new_stestr:
            testtools.TestCase.skipException = unittest.case.SkipTest
        else:
            testtools.TestCase.skipException = unittest2.case.SkipTest
    except Exception:
        pass


class BaseTestCase(testtools.testcase.WithAttributes, testtools.TestCase):
    setUpClassCalled = False

    # NOTE(sdague): log_format is defined inline here instead of using the oslo
    # default because going through the config path recouples config to the
    # stress tests too early, and depending on testr order will fail unit tests
    log_format = ('%(asctime)s %(process)d %(levelname)-8s '
                  '[%(name)s] %(message)s')

    @classmethod
    def setUpClass(cls):
        if hasattr(super(BaseTestCase, cls), 'setUpClass'):
            super(BaseTestCase, cls).setUpClass()
        cls.setUpClassCalled = True
        # TODO(gmann): cls.handle_skip_exception is really workaround for
        # testtools bug- https://github.com/testing-cabal/testtools/issues/272
        # stestr which is used by Tempest internally to run the test switch
        # the customize test runner(which use stdlib unittest) for >=py3.5
        # else testtools.run.- https://github.com/mtreinish/stestr/pull/265
        # These two test runner are not compatible due to skip exception
        # handling(due to unittest2). testtools.run treat unittestt.SkipTest
        # as error and stdlib unittest treat unittest2.case.SkipTest raised
        # by testtools.TestCase.skipException.
        # The below workaround can be removed once testtools fix issue# 272.
        cls.orig_skip_exception = testtools.TestCase.skipException
        _handle_skip_exception()

    @classmethod
    def tearDownClass(cls):
        if hasattr(super(BaseTestCase, cls), 'tearDownClass'):
            super(BaseTestCase, cls).tearDownClass()

    def setUp(self):
        testtools.TestCase.skipException = self.orig_skip_exception
        super(BaseTestCase, self).setUp()
        if not self.setUpClassCalled:
            raise RuntimeError("setUpClass does not calls the super's "
                               "setUpClass in {!r}".format(type(self)))
        test_timeout = os.environ.get('OS_TEST_TIMEOUT', 0)
        try:
            test_timeout = int(test_timeout)
        except ValueError:
            test_timeout = 0
        if test_timeout > 0:
            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

        if (os.environ.get('OS_STDOUT_CAPTURE') == 'True' or
                os.environ.get('OS_STDOUT_CAPTURE') == '1'):
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if (os.environ.get('OS_STDERR_CAPTURE') == 'True' or
                os.environ.get('OS_STDERR_CAPTURE') == '1'):
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))
        if (os.environ.get('OS_LOG_CAPTURE') != 'False' and
                os.environ.get('OS_LOG_CAPTURE') != '0'):
            self.useFixture(fixtures.LoggerFixture(nuke_handlers=False,
                                                   format=self.log_format,
                                                   level=None))



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

from oslo_serialization import jsonutils as json

from tempest.lib.services.volume.v3 import transfers_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestTransfersClient(base.BaseServiceTest):

    FAKE_VOLUME_TRANSFER_ID = "0e89cdd1-6249-421b-96d8-25fac0623d42"

    FAKE_VOLUME_TRANSFER_INFO = {
        "transfer": {
            "id": FAKE_VOLUME_TRANSFER_ID,
            "name": "fake-volume-transfer",
            "volume_id": "47bf04ef-1ea5-4c5f-a375-430a086d6747",
            "created_at": "2017-04-18T09:10:03.000000",
            "links": [
                {
                    "href": "fake-url-1",
                    "rel": "self"
                },
                {
                    "href": "fake-url-2",
                    "rel": "bookmark"
                }
            ]
        }
    }

    def setUp(self):
        super(TestTransfersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = transfers_client.TransfersClient(fake_auth,
                                                       'volume',
                                                       'regionOne')

    def _test_create_volume_transfer(self, bytes_body=False):
        resp_body = copy.deepcopy(self.FAKE_VOLUME_TRANSFER_INFO)
        resp_body['transfer'].update({"auth_key": "fake-auth-key"})
        kwargs = {"name": "fake-volume-transfer",
                  "volume_id": "47bf04ef-1ea5-4c5f-a375-430a086d6747"}
        payload = json.dumps({"transfer": kwargs}, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(transfers_client.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.create_volume_transfer,
                'tempest.lib.common.rest_client.RestClient.post',
                resp_body,
                to_utf=bytes_body,
                status=202,
                mock_args=['os-volume-transfer', payload],
                **kwargs)

    def _test_accept_volume_transfer(self, bytes_body=False):
        resp_body = copy.deepcopy(self.FAKE_VOLUME_TRANSFER_INFO)
        resp_body['transfer'].pop('created_at')
        kwargs = {"auth_key": "fake-auth-key"}
        payload = json.dumps({"accept": kwargs}, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(transfers_client.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.accept_volume_transfer,
                'tempest.lib.common.rest_client.RestClient.post',
                resp_body,
                to_utf=bytes_body,
                status=202,
                mock_args=['os-volume-transfer/%s/accept' %
                           self.FAKE_VOLUME_TRANSFER_ID, payload],
                transfer_id=self.FAKE_VOLUME_TRANSFER_ID,
                **kwargs)

    def _test_show_volume_transfer(self, bytes_body=False):
        resp_body = self.FAKE_VOLUME_TRANSFER_INFO
        self.check_service_client_function(
            self.client.show_volume_transfer,
            'tempest.lib.common.rest_client.RestClient.get',
            resp_body,
            to_utf=bytes_body,
            transfer_id="0e89cdd1-6249-421b-96d8-25fac0623d42")

    def _test_list_volume_transfers(self, detail=False, bytes_body=False):
        resp_body = copy.deepcopy(self.FAKE_VOLUME_TRANSFER_INFO)
        if not detail:
            resp_body['transfer'].pop('created_at')
        resp_body = {"transfers": [resp_body['transfer']]}
        self.check_service_client_function(
            self.client.list_volume_transfers,
            'tempest.lib.common.rest_client.RestClient.get',
            resp_body,
            to_utf=bytes_body,
            detail=detail)

    def test_create_volume_transfer_with_str_body(self):
        self._test_create_volume_transfer()

    def test_create_volume_transfer_with_bytes_body(self):
        self._test_create_volume_transfer(bytes_body=True)

    def test_accept_volume_transfer_with_str_body(self):
        self._test_accept_volume_transfer()

    def test_accept_volume_transfer_with_bytes_body(self):
        self._test_accept_volume_transfer(bytes_body=True)

    def test_show_volume_transfer_with_str_body(self):
        self._test_show_volume_transfer()

    def test_show_volume_transfer_with_bytes_body(self):
        self._test_show_volume_transfer(bytes_body=True)

    def test_list_volume_transfers_with_str_body(self):
        self._test_list_volume_transfers()

    def test_list_volume_transfers_with_bytes_body(self):
        self._test_list_volume_transfers(bytes_body=True)

    def test_list_volume_transfers_with_detail_with_str_body(self):
        self._test_list_volume_transfers(detail=True)

    def test_list_volume_transfers_with_detail_with_bytes_body(self):
        self._test_list_volume_transfers(detail=True, bytes_body=True)

    def test_delete_volume_transfer(self):
        self.check_service_client_function(
            self.client.delete_volume_transfer,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=202,
            transfer_id="0e89cdd1-6249-421b-96d8-25fac0623d42")



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import floating_ips as schema
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import base_compute_client


class FloatingIPsClient(base_compute_client.BaseComputeClient):

    def list_floating_ips(self, **params):
        """Returns a list of all floating IPs filtered by any parameters.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-floating-ip-addresses
        """
        url = 'os-floating-ips'
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_floating_ips, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_floating_ip(self, floating_ip_id):
        """Get the details of a floating IP.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-floating-ip-address-details
        """
        url = "os-floating-ips/%s" % floating_ip_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.create_get_floating_ip, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_floating_ip(self, **kwargs):
        """Allocate a floating IP to the project.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-allocate-floating-ip-address
        """
        url = 'os-floating-ips'
        post_body = json.dumps(kwargs)
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        self.validate_response(schema.create_get_floating_ip, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_floating_ip(self, floating_ip_id):
        """Deletes the provided floating IP from the project.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-deallocate-floating-ip-address
        """
        url = "os-floating-ips/%s" % floating_ip_id
        resp, body = self.delete(url)
        self.validate_response(schema.add_remove_floating_ip, resp, body)
        return rest_client.ResponseBody(resp, body)

    def associate_floating_ip_to_server(self, floating_ip, server_id):
        """Associate the provided floating IP to a specific server."""
        url = "servers/%s/action" % server_id
        post_body = {
            'addFloatingIp': {
                'address': floating_ip,
            }
        }

        post_body = json.dumps(post_body)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.add_remove_floating_ip, resp, body)
        return rest_client.ResponseBody(resp, body)

    def disassociate_floating_ip_from_server(self, floating_ip, server_id):
        """Disassociate the provided floating IP from a specific server."""
        url = "servers/%s/action" % server_id
        post_body = {
            'removeFloatingIp': {
                'address': floating_ip,
            }
        }

        post_body = json.dumps(post_body)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.add_remove_floating_ip, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_floating_ip(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'floating_ip'



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import images as schema
from tempest.lib.api_schema.response.compute.v2_45 import images as schemav245
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import base_compute_client


class ImagesClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.44', 'schema': schema},
        {'min': '2.45', 'max': None, 'schema': schemav245}]

    def create_image(self, server_id, **kwargs):
        """Create an image of the original server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-image-createimage-action
        """

        post_body = {'createImage': kwargs}
        post_body = json.dumps(post_body)
        resp, body = self.post('servers/%s/action' % server_id,
                               post_body)
        _schema = self.get_schema(self.schema_versions_info)
        if body:
            body = json.loads(body)
        self.validate_response(_schema.create_image, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_images(self, detail=False, **params):
        """Return a list of all images filtered by any parameter.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-images
        https://docs.openstack.org/api-ref/compute/#list-images-with-details
        """
        url = 'images'
        _schema = schema.list_images
        if detail:
            url += '/detail'
            _schema = schema.list_images_details

        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(_schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_image(self, image_id):
        """Return the details of a single image."""
        resp, body = self.get("images/%s" % image_id)
        body = json.loads(body)
        self.validate_response(schema.get_image, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_image(self, image_id):
        """Delete the provided image."""
        resp, body = self.delete("images/%s" % image_id)
        self.validate_response(schema.delete, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_image_metadata(self, image_id):
        """List all metadata items for an image."""
        resp, body = self.get("images/%s/metadata" % image_id)
        body = json.loads(body)
        self.validate_response(schema.image_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def set_image_metadata(self, image_id, meta):
        """Set the metadata for an image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-image-metadata
        """
        post_body = json.dumps({'metadata': meta})
        resp, body = self.put('images/%s/metadata' % image_id, post_body)
        body = json.loads(body)
        self.validate_response(schema.image_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_image_metadata(self, image_id, meta):
        """Update the metadata for an image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-image-metadata
        """
        post_body = json.dumps({'metadata': meta})
        resp, body = self.post('images/%s/metadata' % image_id, post_body)
        body = json.loads(body)
        self.validate_response(schema.image_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_image_metadata_item(self, image_id, key):
        """Return the value for a specific image metadata key."""
        resp, body = self.get("images/%s/metadata/%s" % (image_id, key))
        body = json.loads(body)
        self.validate_response(schema.image_meta_item, resp, body)
        return rest_client.ResponseBody(resp, body)

    def set_image_metadata_item(self, image_id, key, meta):
        """Set the value for a specific image metadata key.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-or-update-image-metadata-item
        """
        post_body = json.dumps({'meta': meta})
        resp, body = self.put('images/%s/metadata/%s' % (image_id, key),
                              post_body)
        body = json.loads(body)
        self.validate_response(schema.image_meta_item, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_image_metadata_item(self, image_id, key):
        """Delete a single image metadata key/value pair."""
        resp, body = self.delete("images/%s/metadata/%s" %
                                 (image_id, key))
        self.validate_response(schema.delete, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        # Added status check for user with admin role
        try:
            if self.show_image(id)['image']['status'] == 'DELETED':
                return True
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Return the primary type of resource this client works with."""
        return 'image'



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import matchers

from tempest.api.volume import base
from tempest.common import utils
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class VolumesImageMetadata(base.BaseVolumeTest):
    """Test volume image metadata"""

    @classmethod
    def skip_checks(cls):
        super(VolumesImageMetadata, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as Glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(VolumesImageMetadata, cls).resource_setup()
        # Create a volume from image ID
        cls.volume = cls.create_volume(imageRef=CONF.compute.image_ref)

    @decorators.idempotent_id('03efff0b-5c75-4822-8f10-8789ac15b13e')
    @utils.services('image')
    def test_update_show_delete_image_metadata(self):
        """Test update/show/delete volume's image metadata"""
        # Update image metadata
        image_metadata = {'image_id': '5137a025-3c5f-43c1-bc64-5f41270040a5',
                          'image_name': 'image',
                          'kernel_id': '6ff710d2-942b-4d6b-9168-8c9cc2404ab1',
                          'ramdisk_id': 'somedisk'}
        self.volumes_client.update_volume_image_metadata(self.volume['id'],
                                                         **image_metadata)

        # Fetch volume's image metadata by show_volume method
        volume_image_metadata = self.volumes_client.show_volume(
            self.volume['id'])['volume']['volume_image_metadata']

        # Verify image metadata was updated
        self.assertThat(volume_image_metadata.items(),
                        matchers.ContainsAll(image_metadata.items()))

        # Delete one item from image metadata of the volume
        self.volumes_client.delete_volume_image_metadata(self.volume['id'],
                                                         'ramdisk_id')
        del image_metadata['ramdisk_id']

        # Fetch volume's image metadata by show_volume_image_metadata method
        volume_image_metadata = self.volumes_client.show_volume_image_metadata(
            self.volume['id'])['metadata']

        # Verify image metadata was updated after item deletion
        self.assertThat(volume_image_metadata.items(),
                        matchers.ContainsAll(image_metadata.items()))
        self.assertNotIn('ramdisk_id', volume_image_metadata)



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.api.image import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ImageMembersTest(base.BaseV1ImageMembersTest):
    """Test image members"""

    @decorators.idempotent_id('1d6ef640-3a20-4c84-8710-d95828fdb6ad')
    def test_add_image_member(self):
        """Test adding member for image"""
        image = self._create_image()
        self.image_member_client.create_image_member(image, self.alt_tenant_id)
        body = self.image_member_client.list_image_members(image)
        members = body['members']
        members = [member['member_id'] for member in members]
        self.assertIn(self.alt_tenant_id, members)
        # get image as alt user
        self.alt_img_cli.show_image(image)

    @decorators.idempotent_id('6a5328a5-80e8-4b82-bd32-6c061f128da9')
    def test_get_shared_images(self):
        """Test getting shared images"""
        image = self._create_image()
        self.image_member_client.create_image_member(image, self.alt_tenant_id)
        share_image = self._create_image()
        self.image_member_client.create_image_member(share_image,
                                                     self.alt_tenant_id)
        body = self.image_member_client.list_shared_images(
            self.alt_tenant_id)
        images = body['shared_images']
        images = [img['image_id'] for img in images]
        self.assertIn(share_image, images)
        self.assertIn(image, images)

    @decorators.idempotent_id('a76a3191-8948-4b44-a9d6-4053e5f2b138')
    def test_remove_member(self):
        """Test removing member from image"""
        image_id = self._create_image()
        self.image_member_client.create_image_member(image_id,
                                                     self.alt_tenant_id)
        self.image_member_client.delete_image_member(image_id,
                                                     self.alt_tenant_id)
        body = self.image_member_client.list_image_members(image_id)
        members = body['members']
        self.assertEmpty(members)
        self.assertRaises(
            lib_exc.NotFound, self.alt_img_cli.show_image, image_id)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.volume.v3 import capabilities_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestCapabilitiesClient(base.BaseServiceTest):

    FAKE_BACKEND_CAPABILITIES = {
        "namespace": "OS::Storage::Capabilities::fake",
        "vendor_name": "OpenStack",
        "volume_backend_name": "lvmdriver-1",
        "pool_name": "pool",
        "driver_version": "2.0.0",
        "storage_protocol": "iSCSI",
        "display_name": "Capabilities of Cinder LVM driver",
        "description": (
            "These are volume type options provided by Cinder LVM driver."),
        "visibility": "public",
        "replication_targets": [],
        "properties": {
            "compression": {
                "title": "Compression",
                "description": "Enables compression.",
                "type": "boolean"
            },
            "qos": {
                "title": "QoS",
                "description": "Enables QoS.",
                "type": "boolean"
            },
            "replication": {
                "title": "Replication",
                "description": "Enables replication.",
                "type": "boolean"
            },
            "thin_provisioning": {
                "title": "Thin Provisioning",
                "description": "Sets thin provisioning.",
                "type": "boolean"
            }
        }
    }

    def setUp(self):
        super(TestCapabilitiesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = capabilities_client.CapabilitiesClient(
            fake_auth, 'volume', 'regionOne')

    def _test_show_backend_capabilities(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_backend_capabilities,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_BACKEND_CAPABILITIES,
            bytes_body,
            host='lvmdriver-1')

    def test_show_backend_capabilities_with_str_body(self):
        self._test_show_backend_capabilities()

    def test_show_backend_capabilities_with_bytes_body(self):
        self._test_show_backend_capabilities(bytes_body=True)



# Copyright 2016 Rackspace
#
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
from io import StringIO
import os
import shutil
import subprocess
import sys
import tempfile
from unittest import mock
from unittest.mock import patch


from oslo_serialization import jsonutils as json
from tempest.cmd import subunit_describe_calls
from tempest.tests import base


class TestArgumentParser(base.TestCase):
    def test_init(self):
        test_object = subunit_describe_calls.ArgumentParser()
        self.assertEqual("subunit-describe-calls", test_object.prog)
        self.assertEqual(subunit_describe_calls.DESCRIPTION,
                         test_object.description)


class TestUrlParser(base.TestCase):
    services_custom_ports = {
        "18776": "Block Storage",
        "18774": "Nova",
        "18773": "Nova-API",
        "18386": "Sahara",
        "35358": "Keystone",
        "19292": "Glance",
        "19696": "Neutron",
        "16000": "Swift",
        "18004": "Heat",
        "18777": "Ceilometer",
        "10080": "Horizon",
        "18080": "Swift",
        "1873": "rsync",
        "13260": "iSCSI",
        "13306": "MySQL",
        "15672": "AMQP",
        "18082": "murano"}

    def setUp(self):
        super(TestUrlParser, self).setUp()
        self.test_object = subunit_describe_calls.UrlParser()

    def test_get_service_default_ports(self):
        base_url = "http://site.something.com:"
        for port in self.test_object.services:
            url = base_url + port + "/v2/action"
            service = self.test_object.services[port]
            self.assertEqual(service, self.test_object.get_service(url))

    def test_get_service_custom_ports(self):
        self.test_object = subunit_describe_calls.\
            UrlParser(services=self.services_custom_ports)
        base_url = "http://site.something.com:"
        for port in self.services_custom_ports:
            url = base_url + port + "/v2/action"
            service = self.services_custom_ports[port]
            self.assertEqual(service, self.test_object.get_service(url))

    def test_get_service_port_not_found(self):
        url = "https://site.somewhere.com:1234/v2/action"
        self.assertEqual("Unknown", self.test_object.get_service(url))
        self.assertEqual("Unknown", self.test_object.get_service(""))

    def test_parse_details_none(self):
        self.assertIsNone(self.test_object.parse_details(None))

    def test_url_path_ports(self):
        uuid_sample1 = "3715e0bb-b1b3-4291-aa13-2c86c3b9ec93"
        uuid_sample2 = "2715e0bb-b1b4-4291-aa13-2c86c3b9ec88"

        # test http url
        host = "http://host.company.com"
        url = host + ":8776/v3/" + uuid_sample1 + "/types/" + \
            uuid_sample2 + "/extra_specs"
        self.assertEqual("v3/<uuid>/types/<uuid>/extra_specs",
                         self.test_object.url_path(url))
        url = host + ":8774/v2.1/servers/" + uuid_sample1
        self.assertEqual("v2.1/servers/<uuid>",
                         self.test_object.url_path(url))
        # test https url
        host = "https://host.company.com"
        url = host + ":8776/v3/" + uuid_sample1 + "/types/" + \
            uuid_sample2 + "/extra_specs"
        self.assertEqual("v3/<uuid>/types/<uuid>/extra_specs",
                         self.test_object.url_path(url))
        url = host + ":8774/v2.1/servers/" + uuid_sample1
        self.assertEqual("v2.1/servers/<uuid>",
                         self.test_object.url_path(url))

    def test_url_path_no_match(self):
        host_port = 'https://host.company.com:1234/'
        url = 'v2/action/no/special/data'
        self.assertEqual(url, self.test_object.url_path(host_port + url))
        url = 'data'
        self.assertEqual(url, self.test_object.url_path(url))


class TestCliBase(base.TestCase):
    """Base class for share code on all CLI sub-process testing"""

    def setUp(self):
        super(TestCliBase, self).setUp()
        self._subunit_file = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'subunit_describe_calls_data', 'calls.subunit')

    def _bytes_to_string(self, data):
        if isinstance(data, (bytes, bytearray)):
            data = str(data, 'utf-8')
        return data

    def _assert_cli_message(self, data):
        data = self._bytes_to_string(data)
        self.assertIn("Running subunit_describe_calls ...", data)

    def _assert_deprecated_warning(self, stdout):
        self.assertIn(
            b"Use of: 'subunit-describe-calls' is deprecated, "
            b"please use: 'tempest subunit-describe-calls'", stdout)

    def _assert_expect_json(self, json_data):
        expected_file_name = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'subunit_describe_calls_data', 'calls_subunit_expected.json')
        with open(expected_file_name, "rb") as read_file:
            expected_result = json.load(read_file)
        self.assertDictEqual(expected_result, json_data)

    def _assert_headers_and_bodies(self, data):
        data = self._bytes_to_string(data)
        self.assertIn('- request headers:', data)
        self.assertIn('- request body:', data)
        self.assertIn('- response headers:', data)
        self.assertIn('- response body:', data)

    def _assert_methods_details(self, data):
        data = self._bytes_to_string(data)
        self.assertIn('foo', data)
        self.assertIn('- 200 POST request for Nova to v2.1/<id>/',
                      data)
        self.assertIn('- 200 DELETE request for Nova to v2.1/<id>/',
                      data)
        self.assertIn('- 200 GET request for Nova to v2.1/<id>/',
                      data)
        self.assertIn('- 404 DELETE request for Nova to v2.1/<id>/',
                      data)

    def _assert_mutual_exclusive_message(self, stderr):
        self.assertIn(b"usage: subunit-describe-calls "
                      b"[-h] [-s [<subunit file>]]", stderr)
        self.assertIn(b"[-n <non subunit name>] [-o <output file>]",
                      stderr)
        self.assertIn(b"[-p <ports file>] [-v | -a]", stderr)
        self.assertIn(
            b"subunit-describe-calls: error: argument -v/--verbose: "
            b"not allowed with argument -a/--all-stdout", stderr)

    def _assert_no_headers_and_bodies(self, data):
        data = self._bytes_to_string(data)
        self.assertNotIn('- request headers:', data)
        self.assertNotIn('- request body:', data)
        self.assertNotIn('- response headers:', data)
        self.assertNotIn('- response body:', data)


class TestMainCli(TestCliBase):
    """Test cases that use subunit_describe_calls module main interface

    via subprocess calls to make sure the total user experience
    is well defined and tested. This interface is deprecated.
    Note: these test do not affect code coverage percentages.
    """

    def test_main_output_file(self):
        temp_file = tempfile.mkstemp()[1]
        p = subprocess.Popen([
            'subunit-describe-calls', '-s', self._subunit_file,
            '-o', temp_file], stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_deprecated_warning(stdout)
        with open(temp_file, 'r') as file:
            data = json.loads(file.read())
        self._assert_expect_json(data)

    def test_main_verbose(self):
        p = subprocess.Popen([
            'subunit-describe-calls', '-s', self._subunit_file,
            '-v'], stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_deprecated_warning(stdout)
        self._assert_methods_details(stdout)
        self._assert_headers_and_bodies(stdout)

    def test_main_all_stdout(self):
        p = subprocess.Popen([
            'subunit-describe-calls', '-s', self._subunit_file,
            '--all-stdout'], stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_deprecated_warning(stdout)
        self._assert_methods_details(stdout)
        self._assert_headers_and_bodies(stdout)

    def test_main(self):
        p = subprocess.Popen([
            'subunit-describe-calls', '-s', self._subunit_file],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_deprecated_warning(stdout)
        self._assert_methods_details(stdout)
        self._assert_no_headers_and_bodies(stdout)

    def test_main_verbose_and_all_stdout(self):
        p = subprocess.Popen([
            'subunit-describe-calls', '-s', self._subunit_file,
            '-a', '-v'],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(2, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_deprecated_warning(stdout)
        self._assert_mutual_exclusive_message(stderr)


class TestCli(TestCliBase):
    """Test cases that use tempest subunit_describe_calls cliff interface

    via subprocess calls to make sure the total user experience
    is well defined and tested.
    Note: these test do not affect code coverage percentages.
    """

    def _assert_cliff_verbose(self, stdout):
        self.assertIn(b'tempest initialize_app', stdout)
        self.assertIn(b'prepare_to_run_command TempestSubunitDescribeCalls',
                      stdout)
        self.assertIn(b'tempest clean_up TempestSubunitDescribeCalls',
                      stdout)

    def test_run_all_stdout(self):
        p = subprocess.Popen(['tempest', 'subunit-describe-calls',
                              '-s', self._subunit_file, '-a'],
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_methods_details(stdout)
        self._assert_headers_and_bodies(stdout)

    def test_run_verbose(self):
        p = subprocess.Popen(['tempest', 'subunit-describe-calls',
                              '-s', self._subunit_file, '-v'],
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_methods_details(stdout)
        self._assert_no_headers_and_bodies(stdout)
        self._assert_cliff_verbose(stderr)

    def test_run_min(self):
        p = subprocess.Popen(['tempest', 'subunit-describe-calls',
                              '-s', self._subunit_file],
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_methods_details(stdout)
        self._assert_no_headers_and_bodies(stdout)

    def test_run_verbose_all_stdout(self):
        """Test Cliff -v argument

        Since Cliff framework has a argument at the
        abstract command level the -v or --verbose for
        this command is not processed as a boolean.
        So the use of verbose only exists for the
        deprecated main CLI interface.  When the
        main is deleted this test would not be needed.
        """
        p = subprocess.Popen(['tempest', 'subunit-describe-calls',
                              '-s', self._subunit_file, '-a', '-v'],
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEqual(0, p.returncode)
        self._assert_cli_message(stdout)
        self._assert_cliff_verbose(stderr)
        self._assert_methods_details(stdout)


class TestSubunitDescribeCalls(TestCliBase):
    """Test cases use the subunit_describe_calls module interface

    and effect code coverage reporting
    """

    def setUp(self):
        super(TestSubunitDescribeCalls, self).setUp()
        self.test_object = subunit_describe_calls.TempestSubunitDescribeCalls(
            app=mock.Mock(),
            app_args=mock.Mock(spec=argparse.Namespace))

    def test_parse(self):
        with open(self._subunit_file, 'r') as read_file:
            parser = subunit_describe_calls.parse(
                read_file, "pythonlogging", None)
        self._assert_expect_json(parser.test_logs)

    def test_get_description(self):
        self.assertEqual(subunit_describe_calls.DESCRIPTION,
                         self.test_object.get_description())

    def test_get_parser_default_min(self):
        parser = self.test_object.get_parser('NAME')
        parsed_args = parser.parse_args([])
        self.assertIsNone(parsed_args.output_file)
        self.assertIsNone(parsed_args.ports)
        self.assertFalse(parsed_args.all_stdout)
        self.assertEqual(parsed_args.subunit, sys.stdin)

    def test_get_parser_default_max(self):
        temp_dir = tempfile.mkdtemp(prefix="parser")
        self.addCleanup(shutil.rmtree, temp_dir, ignore_errors=True)
        outfile_name = os.path.join(temp_dir, 'output.json')
        open(outfile_name, 'a').close()
        portfile_name = os.path.join(temp_dir, 'ports.json')
        open(portfile_name, 'a').close()

        parser = self.test_object.get_parser('NAME')
        parsed_args = parser.parse_args(["-a", "-o " + outfile_name,
                                         "-p " + portfile_name])

        self.assertIsNotNone(parsed_args.output_file)
        self.assertIsNotNone(parsed_args.ports)
        self.assertTrue(parsed_args.all_stdout)
        self.assertEqual(parsed_args.subunit, sys.stdin)

    def test_take_action_min(self):
        parser = self.test_object.get_parser('NAME')
        parsed_args = parser.parse_args(["-s" + self._subunit_file],)
        with patch('sys.stdout', new=StringIO()) as mock_stdout:
            self.test_object.take_action(parsed_args)

        stdout_data = mock_stdout.getvalue()
        self._assert_methods_details(stdout_data)
        self._assert_no_headers_and_bodies(stdout_data)

    def test_take_action_all_stdout(self):
        parser = self.test_object.get_parser('NAME')
        parsed_args = parser.parse_args(["-as" + self._subunit_file],)
        with patch('sys.stdout', new=StringIO()) as mock_stdout:
            self.test_object.take_action(parsed_args)

        stdout_data = mock_stdout.getvalue()
        self._assert_methods_details(stdout_data)
        self._assert_headers_and_bodies(stdout_data)

    def test_take_action_outfile_files(self):
        temp_file = tempfile.mkstemp()[1]
        parser = self.test_object.get_parser('NAME')
        parsed_args = parser.parse_args(
            ["-as" + self._subunit_file, '-o', temp_file], )
        with patch('sys.stdout', new=StringIO()) as mock_stdout:
            self.test_object.take_action(parsed_args)
        stdout_data = mock_stdout.getvalue()
        self._assert_cli_message(stdout_data)
        with open(temp_file, 'r') as file:
            data = json.loads(file.read())
        self._assert_expect_json(data)

    def test_take_action_no_items(self):
        temp_file = tempfile.mkstemp()[1]
        parser = self.test_object.get_parser('NAME')
        parsed_args = parser.parse_args(
            ["-as" + temp_file], )
        with patch('sys.stdout', new=StringIO()) as mock_stdout:
            self.test_object.take_action(parsed_args)
        stdout_data = mock_stdout.getvalue()
        self._assert_cli_message(stdout_data)

    def test_take_action_exception(self):
        parser = self.test_object.get_parser('NAME')
        parsed_args = parser.parse_args(["-s" + self._subunit_file],)
        with patch('sys.stderr', new=StringIO()) as mock_stderr:
            with patch('tempest.cmd.subunit_describe_calls.entry_point') \
                    as mock_method:
                mock_method.side_effect = OSError()
                self.assertRaises(OSError, self.test_object.take_action,
                                  parsed_args)
                stderr_data = mock_stderr.getvalue()

        self.assertIn("Traceback (most recent call last):", stderr_data)
        self.assertIn("entry_point(parsed_args)", stderr_data)



# Copyright 2017 AT&T Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json
from six.moves.urllib import parse as urllib

from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class AssistedVolumeSnapshotsClient(base_compute_client.BaseComputeClient):
    """Service client for assisted volume snapshots"""

    def delete_assisted_volume_snapshot(self, volume_id, snapshot_id):
        """Delete snapshot for the given volume id.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-assisted-volume-snapshot

        :param volume_id: UUID of the volume
        :param snapshot_id: The UUID of the snapshot
        """
        query_param = {'delete_info': json.dumps({'volume_id': volume_id})}
        resp, body = self.delete("os-assisted-volume-snapshots/%s?%s"
                                 % (snapshot_id,
                                    urllib.urlencode(query_param)))
        return rest_client.ResponseBody(resp, body)

    def create_assisted_volume_snapshot(self, volume_id, snapshot_id,
                                        **kwargs):
        """Create a new assisted volume snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-assisted-volume-snapshots

        :param volume_id: the source volume ID
        :param snapshot_id: the UUID for a snapshot
        :param type: Type of snapshot, such as qcow2
        :param new_file: The name of image file that will be created
        """
        url = "os-assisted-volume-snapshots"
        info = {"snapshot_id": snapshot_id}
        if kwargs:
            info.update(kwargs)
        body = {"snapshot": {"volume_id": volume_id, "create_info": info}}
        post_body = json.dumps(body)
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools
import sys

import netaddr
from oslo_log import log as logging
import six

from tempest.lib.common import ssh
from tempest.lib.common.utils import test_utils
import tempest.lib.exceptions

LOG = logging.getLogger(__name__)


def debug_ssh(function):
    """Decorator to generate extra debug info in case off SSH failure"""
    @functools.wraps(function)
    def wrapper(self, *args, **kwargs):
        try:
            return function(self, *args, **kwargs)
        except Exception as e:
            caller = test_utils.find_test_caller() or "not found"
            if not isinstance(e, tempest.lib.exceptions.SSHTimeout):
                message = ('Executing command on %(ip)s failed. '
                           'Error: %(error)s' % {'ip': self.ip_address,
                                                 'error': e})
                message = '(%s) %s' % (caller, message)
                LOG.error(message)
                raise
            else:
                try:
                    original_exception = sys.exc_info()
                    if self.server:
                        msg = 'Caller: %s. Timeout trying to ssh to server %s'
                        LOG.debug(msg, caller, self.server)
                        if self.console_output_enabled and self.servers_client:
                            try:
                                msg = 'Console log for server %s: %s'
                                console_log = (
                                    self.servers_client.get_console_output(
                                        self.server['id'])['output'])
                                LOG.debug(msg, self.server['id'], console_log)
                            except Exception:
                                msg = 'Could not get console_log for server %s'
                                LOG.debug(msg, self.server['id'])
                    # re-raise the original ssh timeout exception
                    six.reraise(*original_exception)
                finally:
                    # Delete the traceback to avoid circular references
                    _, _, trace = original_exception
                    del trace
    return wrapper


class RemoteClient(object):

    def __init__(self, ip_address, username, password=None, pkey=None,
                 server=None, servers_client=None, ssh_timeout=300,
                 connect_timeout=60, console_output_enabled=True,
                 ssh_shell_prologue="set -eu -o pipefail; PATH=$PATH:/sbin;",
                 ping_count=1, ping_size=56):
        """Executes commands in a VM over ssh

        :param ip_address: IP address to ssh to
        :param username: Ssh username
        :param password: Ssh password
        :param pkey: Ssh public key
        :param server: Server dict, used for debugging purposes
        :param servers_client: Servers client, used for debugging purposes
        :param ssh_timeout: Timeout in seconds to wait for the ssh banner
        :param connect_timeout: Timeout in seconds to wait for TCP connection
        :param console_output_enabled: Support serial console output?
        :param ssh_shell_prologue: Shell fragments to use before command
        :param ping_count: Number of ping packets
        :param ping_size: Packet size for ping packets
        """
        self.server = server
        self.servers_client = servers_client
        self.ip_address = ip_address
        self.console_output_enabled = console_output_enabled
        self.ssh_shell_prologue = ssh_shell_prologue
        self.ping_count = ping_count
        self.ping_size = ping_size

        self.ssh_client = ssh.Client(ip_address, username, password,
                                     ssh_timeout, pkey=pkey,
                                     channel_timeout=connect_timeout)

    @debug_ssh
    def exec_command(self, cmd):
        # Shell options below add more clearness on failures,
        # path is extended for some non-cirros guest oses (centos7)
        cmd = self.ssh_shell_prologue + " " + cmd
        LOG.debug("Remote command: %s", cmd)
        return self.ssh_client.exec_command(cmd)

    @debug_ssh
    def validate_authentication(self):
        """Validate ssh connection and authentication

           This method raises an Exception when the validation fails.
        """
        self.ssh_client.test_connection_auth()

    def ping_host(self, host, count=None, size=None, nic=None):
        if count is None:
            count = self.ping_count
        if size is None:
            size = self.ping_size

        addr = netaddr.IPAddress(host)
        cmd = 'ping6' if addr.version == 6 else 'ping'
        if nic:
            cmd = 'sudo {cmd} -I {nic}'.format(cmd=cmd, nic=nic)
        cmd += ' -c{0} -w{0} -s{1} {2}'.format(count, size, host)
        return self.exec_command(cmd)

    def mount_config_drive(self):
        """Mount the config drive inside a virtual machine

        This method will not unmount the config drive, so unmount_config_drive
        must be used for cleanup.
        """
        cmd_blkid = 'blkid -L config-2 -o device'
        dev_name = self.exec_command(cmd_blkid).strip()

        try:
            self.exec_command('sudo mount %s /mnt' % dev_name)
        except tempest.lib.exceptions.SSHExecCommandFailed:
            # So the command failed, let's try to know why and print some
            # useful information.
            lsblk = self.exec_command('sudo lsblk --fs --ascii')
            LOG.error("Mounting %s on /mnt failed. Right after the "
                      "failure 'lsblk' in the guest reported:\n%s",
                      dev_name, lsblk)
            raise

    def unmount_config_drive(self):
        self.exec_command('sudo umount /mnt')



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class InstanceActionsNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of instance actions"""

    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(InstanceActionsNegativeTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(InstanceActionsNegativeTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('67e1fce6-7ec2-45c6-92d4-0a8f1a632910')
    def test_list_instance_actions_non_existent_server(self):
        """Test listing actions for non existent instance should fail"""
        non_existent_server_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.list_instance_actions,
                          non_existent_server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0269f40a-6f18-456c-b336-c03623c897f1')
    def test_get_instance_action_invalid_request(self):
        """Test getting instance action with invalid request_id should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.show_instance_action,
                          self.server['id'], '999')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class EndPointGroupsTest(base.BaseIdentityV3AdminTest):
    """Test endpoint groups"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def setup_clients(cls):
        super(EndPointGroupsTest, cls).setup_clients()
        cls.client = cls.endpoint_groups_client

    @classmethod
    def resource_setup(cls):
        super(EndPointGroupsTest, cls).resource_setup()
        cls.endpoint_groups = list()

        # Create endpoint group so as to use it for LIST test
        service_id = cls._create_service()
        cls.addClassResourceCleanup(
            cls.services_client.delete_service, service_id)

        name = data_utils.rand_name('service_group')
        description = data_utils.rand_name('description')
        filters = {'service_id': service_id}

        endpoint_group = cls.client.create_endpoint_group(
            name=name,
            description=description,
            filters=filters)['endpoint_group']
        cls.addClassResourceCleanup(
            cls.client.delete_endpoint_group, endpoint_group['id'])

        cls.endpoint_groups.append(endpoint_group)

    @classmethod
    def _create_service(cls):
        s_name = data_utils.rand_name('service')
        s_type = data_utils.rand_name('type')
        s_description = data_utils.rand_name('description')
        service_data = (
            cls.services_client.create_service(name=s_name,
                                               type=s_type,
                                               description=s_description))

        service_id = service_data['service']['id']
        return service_id

    @decorators.idempotent_id('7c69e7a1-f865-402d-a2ea-44493017315a')
    def test_create_list_show_check_delete_endpoint_group(self):
        """Test create/list/show/check/delete of endpoint group"""
        service_id = self._create_service()
        self.addCleanup(self.services_client.delete_service, service_id)
        name = data_utils.rand_name('service_group')
        description = data_utils.rand_name('description')
        filters = {'service_id': service_id}

        endpoint_group = self.client.create_endpoint_group(
            name=name,
            description=description,
            filters=filters)['endpoint_group']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.client.delete_endpoint_group, endpoint_group['id'])

        self.endpoint_groups.append(endpoint_group)

        # Asserting created endpoint group response body
        self.assertIn('id', endpoint_group)
        self.assertEqual(name, endpoint_group['name'])
        self.assertEqual(description, endpoint_group['description'])

        # Checking if endpoint groups are present in the list of endpoints
        # Note that there are two endpoint groups in the list, one created
        # in the resource setup, one created in this test case.
        fetched_endpoints = \
            self.client.list_endpoint_groups()['endpoint_groups']

        missing_endpoints = \
            [e for e in self.endpoint_groups if e not in fetched_endpoints]

        # Asserting LIST endpoints
        self.assertEmpty(missing_endpoints,
                         "Failed to find endpoint %s in fetched list" %
                         ', '.join(str(e) for e in missing_endpoints))

        # Show endpoint group
        fetched_endpoint = self.client.show_endpoint_group(
            endpoint_group['id'])['endpoint_group']

        # Asserting if the attributes of endpoint group are the same
        self.assertEqual(service_id,
                         fetched_endpoint['filters']['service_id'])
        for attr in ('id', 'name', 'description'):
            self.assertEqual(endpoint_group[attr], fetched_endpoint[attr])

        # Check endpoint group
        self.client.check_endpoint_group(endpoint_group['id'])

        # Deleting the endpoint group created in this method
        self.client.delete_endpoint_group(endpoint_group['id'])

        # Checking whether endpoint group is deleted successfully
        fetched_endpoints = \
            self.client.list_endpoint_groups()['endpoint_groups']
        fetched_endpoint_ids = [e['id'] for e in fetched_endpoints]
        self.assertNotIn(endpoint_group['id'], fetched_endpoint_ids)

    @decorators.idempotent_id('51c8fc38-fa84-4e76-b5b6-6fc37770fb26')
    def test_update_endpoint_group(self):
        """Test updating endpoint group"""
        # Creating an endpoint group so as to check update endpoint group
        # with new values
        service1_id = self._create_service()
        self.addCleanup(self.services_client.delete_service, service1_id)
        name = data_utils.rand_name('service_group')
        description = data_utils.rand_name('description')
        filters = {'service_id': service1_id}

        endpoint_group = self.client.create_endpoint_group(
            name=name,
            description=description,
            filters=filters)['endpoint_group']
        self.addCleanup(self.client.delete_endpoint_group,
                        endpoint_group['id'])

        # Creating new attr values to update endpoint group
        service2_id = self._create_service()
        self.addCleanup(self.services_client.delete_service, service2_id)
        name2 = data_utils.rand_name('service_group2')
        description2 = data_utils.rand_name('description2')
        filters = {'service_id': service2_id}

        # Updating endpoint group with new attr values
        updated_endpoint_group = self.client.update_endpoint_group(
            endpoint_group['id'],
            name=name2,
            description=description2,
            filters=filters)['endpoint_group']

        self.assertEqual(name2, updated_endpoint_group['name'])
        self.assertEqual(description2, updated_endpoint_group['description'])
        self.assertEqual(service2_id,
                         updated_endpoint_group['filters']['service_id'])



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import testtools

from tempest import config
from tempest.lib import exceptions
from tempest.tests import base
from tempest.tests import fake_config


class TestServiceClientConfig(base.TestCase):

    expected_common_params = set(['disable_ssl_certificate_validation',
                                  'ca_certs', 'trace_requests'])
    expected_extra_params = set(['service', 'endpoint_type', 'region',
                                 'build_timeout', 'build_interval'])

    def setUp(self):
        super(TestServiceClientConfig, self).setUp()
        self.useFixture(fake_config.ServiceClientsConfigFixture())
        self.patchobject(config, 'CONF',
                         fake_config.ServiceClientsFakePrivate())
        self.CONF = config.CONF

    def test_service_client_config_no_service(self):
        params = config.service_client_config()
        for param_name in self.expected_common_params:
            self.assertIn(param_name, params)
        for param_name in self.expected_extra_params:
            self.assertNotIn(param_name, params)
        self.assertEqual(
            self.CONF.identity.disable_ssl_certificate_validation,
            params['disable_ssl_certificate_validation'])
        self.assertEqual(self.CONF.identity.ca_certificates_file,
                         params['ca_certs'])
        self.assertEqual(self.CONF.debug.trace_requests,
                         params['trace_requests'])

    def test_service_client_config_service_all(self):
        params = config.service_client_config(
            service_client_name='fake-service1')
        for param_name in self.expected_common_params:
            self.assertIn(param_name, params)
        for param_name in self.expected_extra_params:
            self.assertIn(param_name, params)
        self.assertEqual(self.CONF.fake_service1.catalog_type,
                         params['service'])
        self.assertEqual(self.CONF.fake_service1.endpoint_type,
                         params['endpoint_type'])
        self.assertEqual(self.CONF.fake_service1.region, params['region'])
        self.assertEqual(self.CONF.fake_service1.build_timeout,
                         params['build_timeout'])
        self.assertEqual(self.CONF.fake_service1.build_interval,
                         params['build_interval'])

    def test_service_client_config_service_minimal(self):
        params = config.service_client_config(
            service_client_name='fake-service2')
        for param_name in self.expected_common_params:
            self.assertIn(param_name, params)
        for param_name in self.expected_extra_params:
            self.assertIn(param_name, params)
        self.assertEqual(self.CONF.fake_service2.catalog_type,
                         params['service'])
        self.assertEqual(self.CONF.fake_service2.endpoint_type,
                         params['endpoint_type'])
        self.assertEqual(self.CONF.identity.region, params['region'])
        self.assertEqual(self.CONF.compute.build_timeout,
                         params['build_timeout'])
        self.assertEqual(self.CONF.compute.build_interval,
                         params['build_interval'])

    def test_service_client_config_service_unknown(self):
        unknown_service = 'unknown_service'
        with testtools.ExpectedException(exceptions.UnknownServiceClient,
                                         '.*' + unknown_service + '.*'):
            config.service_client_config(service_client_name=unknown_service)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.api.image import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class CreateDeleteImagesNegativeTest(base.BaseV1ImageTest):
    """Here are negative tests for the deletion and creation of images."""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('036ede36-6160-4463-8c01-c781eee6369d')
    def test_register_with_invalid_container_format(self):
        """Create image with invalid container format

        Negative tests for invalid data supplied to POST /images
        """
        self.assertRaises(lib_exc.BadRequest, self.client.create_image,
                          headers={'x-image-meta-name': 'test',
                                   'x-image-meta-container_format': 'wrong',
                                   'x-image-meta-disk_format': 'vhd'})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('993face5-921d-4e84-aabf-c1bba4234a67')
    def test_register_with_invalid_disk_format(self):
        """Create image with invalid disk format"""
        self.assertRaises(lib_exc.BadRequest, self.client.create_image,
                          headers={'x-image-meta-name': 'test',
                                   'x-image-meta-container_format': 'bare',
                                   'x-image-meta-disk_format': 'wrong'})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ec652588-7e3c-4b67-a2f2-0fa96f57c8fc')
    def test_delete_non_existent_image(self):
        """Return an error while trying to delete a non-existent image"""

        non_existent_image_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          non_existent_image_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('04f72aa3-fcec-45a3-81a3-308ef7cc82bc')
    def test_delete_image_blank_id(self):
        """Return an error while trying to delete an image with blank Id"""
        self.assertRaises(lib_exc.NotFound, self.client.delete_image, '')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('950e5054-a3c7-4dee-ada5-e576f1087abd')
    def test_delete_image_non_hex_string_id(self):
        """Return an error while trying to delete an image with non hex id"""
        invalid_image_id = data_utils.rand_uuid()[:-1] + "j"
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          invalid_image_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('4ed757cd-450c-44b1-9fd1-c819748c650d')
    def test_delete_image_negative_image_id(self):
        """Return an error while trying to delete an image with negative id"""
        self.assertRaises(lib_exc.NotFound, self.client.delete_image, -1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a4a448ab-3db2-4d2d-b9b2-6a1271241dfe')
    def test_delete_image_id_over_character_limit(self):
        """Return an error while trying to delete image with id over limit"""
        overlimit_image_id = data_utils.rand_uuid() + "1"
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          overlimit_image_id)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class VolumeTypesNegativeTest(base.BaseVolumeAdminTest):
    """Negative tests of volume type"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('878b4e57-faa2-4659-b0d1-ce740a06ae81')
    def test_create_with_empty_name(self):
        """Test creating volume type with an empty name will fail"""
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_volume_types_client.create_volume_type, name='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('994610d6-0476-4018-a644-a2602ef5d4aa')
    def test_get_nonexistent_type_id(self):
        """Test getting volume type with nonexistent type id will fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.admin_volume_types_client.show_volume_type,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6b3926d2-7d73-4896-bc3d-e42dfd11a9f6')
    def test_delete_nonexistent_type_id(self):
        """Test deleting volume type with nonexistent type id will fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.admin_volume_types_client.delete_volume_type,
                          data_utils.rand_uuid())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8c09f849-f225-4d78-ba87-bffd9a5e0c6f')
    def test_create_volume_with_private_volume_type(self):
        """Test creating volume with private volume type will fail"""
        params = {'os-volume-type-access:is_public': False}
        volume_type = self.create_volume_type(**params)
        self.assertRaises(lib_exc.NotFound,
                          self.create_volume, volume_type=volume_type['id'])



# Copyright 2017 AT&T Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.identity.v3 import endpoint_filter_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestEndPointsFilterClient(base.BaseServiceTest):
    FAKE_LIST_PROJECTS_FOR_ENDPOINTS = {
        "projects": [
            {
                "domain_id": "1777c7",
                "enabled": True,
                "id": "1234ab1",
                "type": "compute",
                "links": {
                    "self": "http://example.com/identity/v3/projects/1234ab1"
                },
                "name": "Project 1",
                "description": "Project 1 description",
            },
            {
                "domain_id": "1777c7",
                "enabled": True,
                "id": "5678cd2",
                "type": "compute",
                "links": {
                    "self": "http://example.com/identity/v3/projects/5678cd2"
                },
                "name": "Project 2",
                "description": "Project 2 description",
            }
        ],
        "links": {
            "self": "http://example.com/identity/v3/OS-EP-FILTER/endpoints/\
                    u6ay5u/projects",
            "previous": None,
            "next": None
        }
    }

    FAKE_LIST_ENDPOINTS_FOR_PROJECTS = {
        "endpoints": [
            {
                "id": "u6ay5u",
                "interface": "public",
                "url": "http://example.com/identity/",
                "region": "north",
                "links": {
                    "self": "http://example.com/identity/v3/endpoints/u6ay5u"
                },
                "service_id": "5um4r",
            },
            {
                "id": "u6ay5u",
                "interface": "internal",
                "url": "http://example.com/identity/",
                "region": "south",
                "links": {
                    "self": "http://example.com/identity/v3/endpoints/u6ay5u"
                },
                "service_id": "5um4r",
            },
        ],
        "links": {
            "self": "http://example.com/identity/v3/OS-EP-FILTER/projects/\
                    1234ab1/endpoints",
            "previous": None,
            "next": None
        }
    }

    FAKE_LIST_ENDPOINT_GROUPS_FOR_PROJECT = {
        "endpoint_groups": [
            {
                "endpoint_group": {
                    "description": "endpoint group description #2",
                    "filters": {
                        "interface": "admin"
                    },
                    "id": "3de68c",
                    "name": "endpoint group name #2"
                }
            }
            ],
        "links": {
            "self": "https://url/identity/v3/OS-EP-FILTER/endpoint_groups",
        }
    }

    FAKE_PROJECT_INFO = {
        "project": {
            "domain_id": "1789d1",
            "id": "263fd9",
            "links": {
                "self": "http://example.com/identity/v3/projects/263fd9"
            },
            "name": "project name #1",
            "description": "project description #1"
        }
    }

    def setUp(self):
        super(TestEndPointsFilterClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = endpoint_filter_client.EndPointsFilterClient(
            fake_auth, 'identity', 'regionOne')

    def _test_add_endpoint_to_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.add_endpoint_to_project,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            status=204,
            project_id=3,
            endpoint_id=4)

    def _test_check_endpoint_in_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.check_endpoint_in_project,
            'tempest.lib.common.rest_client.RestClient.head',
            {},
            bytes_body,
            status=204,
            project_id=3,
            endpoint_id=4)

    def _test_list_projects_for_endpoint(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_projects_for_endpoint,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_PROJECTS_FOR_ENDPOINTS,
            bytes_body,
            status=200,
            endpoint_id=3)

    def _test_list_endpoints_in_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_endpoints_in_project,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ENDPOINTS_FOR_PROJECTS,
            bytes_body,
            status=200,
            project_id=4)

    def _test_delete_endpoint_from_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.delete_endpoint_from_project,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            bytes_body,
            status=204,
            project_id=3,
            endpoint_id=4)

    def _test_list_endpoint_groups_for_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_endpoint_groups_for_project,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ENDPOINT_GROUPS_FOR_PROJECT,
            bytes_body,
            status=200,
            project_id=3)

    def _test_list_projects_for_endpoint_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_projects_for_endpoint_group,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_PROJECTS_FOR_ENDPOINTS,
            bytes_body,
            status=200,
            endpoint_group_id=5)

    def _test_list_endpoints_for_endpoint_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_endpoints_for_endpoint_group,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ENDPOINTS_FOR_PROJECTS,
            bytes_body,
            status=200,
            endpoint_group_id=5)

    def _test_add_endpoint_group_to_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.add_endpoint_group_to_project,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            status=204,
            endpoint_group_id=5,
            project_id=6)

    def _test_show_endpoint_group_for_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_endpoint_group_for_project,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_PROJECT_INFO,
            bytes_body,
            endpoint_group_id=5,
            project_id=6)

    def test_add_endpoint_to_project_with_str_body(self):
        self._test_add_endpoint_to_project()

    def test_add_endpoint_to_project_with_bytes_body(self):
        self._test_add_endpoint_to_project(bytes_body=True)

    def test_check_endpoint_in_project_with_str_body(self):
        self._test_check_endpoint_in_project()

    def test_check_endpoint_in_project_with_bytes_body(self):
        self._test_check_endpoint_in_project(bytes_body=True)

    def test_list_projects_for_endpoint_with_str_body(self):
        self._test_list_projects_for_endpoint()

    def test_list_projects_for_endpoint_with_bytes_body(self):
        self._test_list_projects_for_endpoint(bytes_body=True)

    def test_list_endpoints_in_project_with_str_body(self):
        self._test_list_endpoints_in_project()

    def test_list_endpoints_in_project_with_bytes_body(self):
        self._test_list_endpoints_in_project(bytes_body=True)

    def test_delete_endpoint_from_project(self):
        self._test_delete_endpoint_from_project()

    def test_list_endpoint_groups_for_project_with_str_body(self):
        self._test_list_endpoint_groups_for_project()

    def test_list_endpoint_groups_for_project_with_bytes_body(self):
        self._test_list_endpoint_groups_for_project(bytes_body=True)

    def test_list_projects_for_endpoint_group_with_str_body(self):
        self._test_list_projects_for_endpoint_group()

    def test_list_projects_for_endpoint_group_with_bytes_body(self):
        self._test_list_projects_for_endpoint_group(bytes_body=True)

    def test_list_endpoints_for_endpoint_group_with_str_body(self):
        self._test_list_endpoints_for_endpoint_group()

    def test_list_endpoints_for_endpoint_group_with_bytes_body(self):
        self._test_list_endpoints_for_endpoint_group(bytes_body=True)

    def test_add_endpoint_group_to_project_with_str_body(self):
        self._test_add_endpoint_group_to_project()

    def test_add_endpoint_group_to_project_with_bytes_body(self):
        self._test_add_endpoint_group_to_project(bytes_body=True)

    def test_show_endpoint_group_for_project_with_str_body(self):
        self._test_show_endpoint_group_for_project()

    def test_show_endpoint_group_for_project_with_bytes_body(self):
        self._test_show_endpoint_group_for_project(bytes_body=True)

    def test_delete_endpoint_group_from_project(self):
        self.check_service_client_function(
            self.client.delete_endpoint_group_from_project,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            False,
            status=204,
            endpoint_group_id=5,
            project_id=5)



# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.image import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


class ImagesTagsTest(base.BaseV2ImageTest):
    """Test image tags"""

    @decorators.idempotent_id('10407036-6059-4f95-a2cd-cbbbee7ed329')
    def test_update_delete_tags_for_image(self):
        """Test adding and deleting image tags"""
        image = self.create_image(container_format='bare',
                                  disk_format='raw',
                                  visibility='private')
        tag = data_utils.rand_name('tag')
        self.addCleanup(self.client.delete_image, image['id'])

        # Creating image tag and verify it.
        self.client.add_image_tag(image['id'], tag)
        body = self.client.show_image(image['id'])
        self.assertIn(tag, body['tags'])

        # Deleting image tag and verify it.
        self.client.delete_image_tag(image['id'], tag)
        body = self.client.show_image(image['id'])
        self.assertNotIn(tag, body['tags'])
